--
-- demonstrator for Lua binding on pixwerx
--

pw.script:load_all()

local w = pw.window.new()
w.visible = false

-- set title
w.title = "My Cool 3D Content"

-- set size
w.size = pw.size.new(640,480)

-- move window
w.position = pw.point.new(100,100)

-- create a new geometry
local g = pw.geometry:new()

g.topology = pw.geometry.triangles -- meh

g.vertices:clear()

z = -5.0
s = 2

g.vertices:add(pw.vector3:new(-s, s, z)) -- 0
g.vertices:add(pw.vector3:new(-s,-s, z)) -- 1
g.vertices:add(pw.vector3:new( s,-s, z)) -- 2
g.vertices:add(pw.vector3:new( s, s, z)) -- 3

-- 0 --- 3
-- |  \  |
-- 1 --- 2

g.indices:add(0)
g.indices:add(1)
g.indices:add(2)

g.indices:add(2)
g.indices:add(3)
g.indices:add(0)


local mm = pw.matrix4x4:new()
local mv = pw.matrix4x4:new()
local mp = pw.matrix4x4:new()



local s = pw.shader:new()


s:set_source(pw.shader_type.vertex,[[
#version 400
uniform mat4 model;
uniform mat4 view;
uniform mat4 projection;

in vec3 vertex_p;

void main() {
    gl_Position = projection * view * model * vec4(vertex_p, 1.0);
}
]])

s:set_source(pw.shader_type.fragment,[[
#version 400
uniform vec4 color = vec4(1.0, 0.0, 0.0, 1.0);
out vec4 frag_colour;
void main() {
    frag_colour = color;
}
]])

if not s:build() then
    print("Error!")
end

local renderer = pw.mesh_renderer:new()

if not renderer:create(g) then
    print("couldn't create renderer")
end

local cam_z = 0
local cam_x = 0

w.on_resize = function(self)
    print(self.client_size.width,self.client_size.height)
end

-- setup a lua callback function as callback
w.on_update = function(self)

    -- set identity on model matrix
    mm:set_identity()
    -- set view matrix with look_at
    mv = pw.matrixtransform.look_at(pw.vector3:new(cam_x,0,cam_z),pw.vector3.forward(),pw.vector3.up())

    -- compute aspect ratio from canvas size
    aspect_ratio = self.client_size.width / self.client_size.height

    -- create a view frustom for a perspective projection
    mp = pw.matrixtransform.perspective_projection(1.3,aspect_ratio,0.2,100)

    -- just some toying around
    local color_red  = pw.mathf.ping_pong(pw.time.now,1.0)
    local color_blue = 1.0 - pw.mathf.ping_pong(pw.time.now,1.0)
    local cl = pw.vector4:new( color_red, 0.0, color_blue, 1.0 )

    -- bind the shader
    s:use()

    s:set_uniform_mat4("model",mm)
    s:set_uniform_mat4("view",mv)
    s:set_uniform_mat4("projection",mp)
    s:set_uniform_vec4("color",cl)

    -- draw
    renderer:draw()

end

-- before entering the update loop make the window visible
w.visible = true

-- main update loop
while w:update() do

    -- somehow works
    if pw.input.get().input_string == 'f' then
        w.fullscreen = not w.fullscreen
    end

    -- keycode for quit
    if pw.input.get().input_string == 'q' then
        break;
    end

    -- just to quickly modify speed
    local move_step = 0.05

    -- camera
    if     pw.input.get().input_string == 'w' then
        cam_z = cam_z - move_step
    elseif pw.input.get().input_string == 's' then
        cam_z = cam_z + move_step
        print(cam_z)
    elseif pw.input.get().input_string == 'a' then
        cam_x = cam_x - move_step
    elseif pw.input.get().input_string == 'd' then
        cam_x = cam_x + move_step
        print(cam_x)
    end

    -- just to check
    if pw.input:get().mouse_button == 1 then

        print("elapsed",t.elapsed)
            t:reset()
        print(pw.input:get().mouse_position.x,pw.input:get().mouse_position.y,w.client_size.width,w.client_size.height)
    end

end
